import re
import shutil
import subprocess
import sys
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import Generator, List, Optional, Union, cast
from xml.dom import minidom


def platform() -> Optional[str]:
    return sys.platform


def execute(cmd) -> Generator[str, None, None]:
    with subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        universal_newlines=True,
        text=True,
        close_fds=True,
        shell=True,
    ) as proc:
        for line in iter(proc.stdout.readline, b""):
            yield line
            if (r := proc.poll()) is not None:
                if r != 0:
                    raise subprocess.CalledProcessError(r, cmd)
                break


def run_sh(cmd: str) -> str:
    # Will raise a CalledProcessError if non-zero return code
    return subprocess.check_output(cmd, text=True, shell=True).strip()


def copy(source: Path, destination: Path, recursive: bool = False):
    if recursive:
        shutil.copytree(source, destination)
    else:
        shutil.copy2(source, destination, follow_symlinks=True)


def move(source: Path, destination: Path):
    shutil.move(source, destination)


def remove(dest: Path):
    shutil.rmtree(dest)


def list_set_intersection(a: list, b: list) -> list:
    return list(set(a) & set(b))


def list_loop_intersection(a: list, b: list) -> list:
    return [value for value in a if value in b]


def list_loop_exclusion(a: list, b: list) -> list:
    return [value for value in a if value not in b]


def list_grab(element: str, root: ET.Element) -> Optional[List[str]]:
    try:
        return cast(
            Optional[List[str]],
            [n.text for n in cast(ET.Element, root.find(element)).findall("li")],
        )
    except AttributeError:
        return None


def element_grab(element: str, root: ET.Element) -> Optional[str]:
    try:
        return cast(ET.Element, root.find(element)).text
    except AttributeError:
        return None


def et_pretty_xml(root: ET.Element) -> str:
    return minidom.parseString(
        re.sub(
            r"[\n\t\s]*",
            "",
            (ET.tostring(cast(ET.Element, root), "utf-8").decode()),
        )
    ).toprettyxml(indent="  ", newl="\n")


def sanitize_path(path: Union[str, Path]):
    if isinstance(path, Path):
        path = str(path)

    if platform() == "win32":
        path.replace('"', "")

    return Path(path).expanduser()

def extract_download_path() -> str:
    '''
    Get the path on where steamcmd downloads items from the query function
    This should prevent that bug where steamcmd downloads to a path different
    that the one that's hardcoded
    '''
    output: str = run_sh('env HOME="/tmp/rmm-dltest" steamcmd +login anonymous +workshop_download_item 294100 2009463077 +quit')
    regex = re.compile(r'\".*rmm-dltest\/([\w\/\.]+294100).*\"')
    match = regex.search(output).group(1)
    return match




#!/usr/bin/env python3

import curses


class WindowSizeException(Exception):
    pass


class AbortModOrderException(Exception):
    pass


def multiselect_order_menu(stdscr, data):
    data = [(n.packageid, n.enabled) for n in data]
    k = 0

    # Clear and refresh the screen for a blank canvas
    stdscr.clear()
    stdscr.refresh()

    # Start colors in curses
    curses.start_color()
    curses.init_pair(1, curses.COLOR_CYAN, curses.COLOR_BLACK)
    curses.init_pair(2, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.init_pair(3, curses.COLOR_BLACK, curses.COLOR_WHITE)

    selection = 0
    window_height, window_width = stdscr.getmaxyx()
    scroll_window_height = window_height - 10
    scroll_window_position = 0

    if window_width < 40 or window_height < 15:
        raise WindowSizeException()

    def check_bounds(location, data, delta_position):
        if delta_position > 0:
            if location < len(data) - 1:
                return True
        if delta_position < 0:
            if location > 0:
                return True
        return False

    def move_selection(selected, l, d):
        if d > 0 and check_bounds(selected, l, d):
            return selected + 1
        if d < 0:
            if selected > 0 and check_bounds(selected, l, d):
                return selected - 1
        return selected

    def list_swap(data, offset, pos):
        temp = data[offset], data[offset + pos]
        data[offset + pos] = temp[0]
        data[offset] = temp[1]

    while True:
        # Initialization
        stdscr.clear()
        window_height, window_width = stdscr.getmaxyx()

        if k == curses.KEY_DOWN:
            selection = move_selection(selection, data, 1)
        elif k == curses.KEY_UP:
            selection = move_selection(selection, data, -1)
        elif k == ord("j"):
            if check_bounds(selection, data, 1):
                list_swap(data, selection, 1)
                selection = move_selection(selection, data, 1)
        elif k == ord("k"):
            if check_bounds(selection, data, -1):
                list_swap(data, selection, -1)
                selection = move_selection(selection, data, -1)
        elif k == curses.KEY_ENTER or k == 10 or k == 13:
            data[selection] = (data[selection][0], not data[selection][1])
            selection = move_selection(selection, data, 1)

        elif k == ord("c"):
            return data

        elif k == ord("q"):
            raise AbortModOrderException()

        if scroll_window_position < len(data) - 1 and selection > (
            scroll_window_position + scroll_window_height - 1
        ):
            scroll_window_position += 1
        if scroll_window_position > 0 and selection < (scroll_window_position):
            scroll_window_position -= 1

        statusbarstr = "Press 'c' to accept changes, 'q' to exit without saving, 'Enter' to enable/disable, 'j/k' to change order."

        max_length = 0
        for n in data:
            if len(n[0]) > max_length:
                max_length = len(n[0])

        scroll_centering_value = (
            scroll_window_height if scroll_window_height < len(data) else len(data)
        )
        start_y = int(
            (window_height // 2)
            - int(scroll_centering_value) // 2
            - int(scroll_centering_value) % 2
        )
        start_x = int((window_width // 2) - int(max_length) // 2)

        for i, (k, v) in enumerate(
            data[scroll_window_position : scroll_window_position + scroll_window_height]
        ):
            if v == False:
                stdscr.addstr(start_y, start_x - 3, "-")
            if v == True:
                stdscr.addstr(start_y, start_x - 3, "+")
            if selection == i + scroll_window_position:
                stdscr.attron(curses.A_STANDOUT)
            stdscr.addstr(start_y, start_x, k)
            if selection == i + scroll_window_position:
                stdscr.attroff(curses.A_STANDOUT)
            start_y += 1

        # Rendering some text
        title = "RMM: Mod Sorting Display"
        stdscr.addstr(0, 0, title, curses.color_pair(1))

        # Render status bar
        stdscr.attron(curses.color_pair(3))
        stdscr.addstr(window_height - 1, 0, statusbarstr[0 : window_width - 1])
        if window_width > len(statusbarstr):
            stdscr.addstr(
                window_height - 1,
                len(statusbarstr),
                " " * (window_width - len(statusbarstr) - 1),
            )
        stdscr.attroff(curses.color_pair(3))

        # Refresh the screen
        stdscr.refresh()

        # Wait for next input
        k = stdscr.getch()


def main():
    text = [
        ("jaxe.rimhud", True),
        ("fluffies.desirepaths", False),
        ("i eat bagels", False),
        ("chonky.cats", False),
    ]
    print(curses.wrapper(multiselect_order_menu, text))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from pathlib import Path
from typing import List, cast
from xml.etree import ElementTree as ET

from . import util
from .mod import EXPANSION_PACKAGES, Mod


class ModsConfig:
    def __init__(self, p: Path):
        if isinstance(p, str):
            p = Path(p)
        self.path = p.expanduser()
        try:
            self.element_tree = ET.parse(self.path)
        except OSError:
            print("Unable to read ModsConfig file at " + str(self.path))
            raise

        self.root = self.element_tree.getroot()
        try:
            enabled = cast(
                List[str],
                util.list_grab("activeMods", self.root),
            )
            # self.mods = [Mod(packageid=pid) for pid in enabled]
            self.mods = {pid: None for pid in enabled}
        except TypeError:
            print("Unable to parse activeMods in ModsConfig")
            raise
        self.version = util.element_grab("version", self.root)
        self.length = len(self.mods)
        try:
            self.expansions = util.list_grab("knownExpansions", self.root)
            if not self.expansions:
                self.expansions = []
        except TypeError:
            self.expansions = []

    def write(self):
        active_mods = self.root.find("activeMods")
        if not active_mods:
            return
        try:
            for item in list(active_mods.findall("li")):
                if active_mods:
                    active_mods.remove(item)
        except AttributeError:
            pass

        try:
            for mod in self.mods:
                new_element = ET.SubElement(active_mods, "li")
                new_element.text = mod
        except AttributeError:
            raise Exception("Unable to find 'activeMods' in ModsConfig")

        buffer = util.et_pretty_xml(self.root)

        try:
            with self.path.open("w+") as f:
                f.seek(0)
                f.write(buffer)
        except OSError:
            print("Unable to write ModsConfig")
            raise

    def enable_mod(self, m: Mod):
        self.mods[m.packageid] = None

    def disable_mod(self, m: Mod):
        if m.packageid in self.mods:
            del self.mods[m.packageid]

    def autosort(self, mods, config):
        import json

        import networkx as nx
        import rmm.manager as manager

        DG = nx.DiGraph()

        before_core = ["brrainz.harmony", "me.samboycoding.betterloading"]

        core = ["ludeon.rimworld"]

        expansion_load_order = [
            "ludeon.rimworld.royalty",
            "ludeon.rimworld.ideology",
        ]

        combined_load_order = before_core + core + expansion_load_order
        for n, pid in enumerate(combined_load_order):
            if not pid in self.mods:
                del combined_load_order[n]

        for k in range(0, len(combined_load_order)):
            for j in range(k + 1, len(combined_load_order)):
                DG.add_edge(combined_load_order[j], combined_load_order[k])

        populated_mods = {m.packageid: m for m in mods if m in self.mods}

        rules_path = Path(
            config.mod_path / "rupal.rimpymodmanagerdatabase/db/communityRules.json"
        )
        if not rules_path.is_file():
            print("Downloading rules file\n")
            # import rmm.steam
            # rules_cache_path = Path(SteamDownloader.find_path()[1]/ "1847679158")
            # rmm.steam.SteamDownloader.download([1847679158])

            manager.Manager(config).sync_mods([Mod(steamid=1847679158)])

        with (
            config.mod_path / "rupal.rimpymodmanagerdatabase/db/communityRules.json"
        ).open("r", encoding="utf-8") as f:
            community_db = json.load(f)

        for pid, m in populated_mods.items():
            try:
                for j in community_db["rules"][m.packageid]["loadAfter"]:
                    if j:
                        try:
                            m.before.add(j)
                        except AttributeError:
                            m.before = set(j)
            except KeyError:
                pass
            try:
                for j in community_db["rules"][m.packageid]["loadBefore"]:
                    if j:
                        try:
                            m.after.add(j)
                        except AttributeError:
                            m.after = set(j)
            except KeyError:
                pass

        rocketman = False
        if "krkr.rocketman" in populated_mods:
            rocketman = True

        if (
            "murmur.walllight" in populated_mods
            and "juanlopez2008.lightsout" in populated_mods
        ):
            DG.add_edge("juanlopez2008.lightsout", "murmur.walllight")

        mods_for_removal = {
            n
            for n in expansion_load_order + before_core
            if n not in EXPANSION_PACKAGES + mods or n not in self.mods
        }

        for pid, m in populated_mods.items():
            if rocketman and pid != "krkr.rocketman":
                DG.add_edge("krkr.rocketman", pid)
            if not m in combined_load_order:
                for n in combined_load_order:
                    DG.add_edge(pid, n)
            if m.after:
                for a in m.after:
                    if a in self.mods:
                        DG.add_edge(a.lower(), pid)
            if m.before:
                for b in m.before:
                    if b in self.mods:
                        DG.add_edge(pid, b.lower())

        count = 0
        while True:
            try:
                sorted_mods = reversed(list(nx.topological_sort(DG)))
                self.mods = [n for n in sorted_mods]
                self.mods = util.list_loop_exclusion(self.mods, mods_for_removal)
                print("Auto-sort complete")

                print(
                    "Verifying state: {}".format(
                        "good" if self.verify_state(populated_mods) else "bad"
                    )
                )
                return
            except nx.exception.NetworkXUnfeasible:
                if count >= 10:
                    print("Unable to break cycles")
                    exit(0)
                print("Cycle found. Breaking load order cycle")
                cycle = nx.find_cycle(DG)
                print(cycle)
                DG.remove_edge(*cycle[0])
                count += 1

    def verify_state(self, mods: List[Mod]):
        if isinstance(mods, list):
            populated_mods = {m.packageid: m for m in mods if m.packageid in self.mods}
        elif isinstance(mods, dict):
            populated_mods = {
                m.packageid: m for m in mods.values() if m.packageid in self.mods
            }
        else:
            raise Exception("bad data type")

        incompatible = dict()
        for m in populated_mods.values():
            try:
                for n in m.incompatible:
                    incompatible[n] = None
            except TypeError:
                pass

        for n in populated_mods:
            if n in incompatible:
                return False

        return True



#!/usr/bin/env python3

import os
import re
import subprocess
import tempfile
import urllib.error
import urllib.request
import zipfile
from pathlib import Path
from typing import List, Tuple

from bs4 import BeautifulSoup

from . import util
from .mod import Mod, ModFolder

STEAMCMD_WINDOWS_URL = "https://steamcdn-a.akamaihd.net/client/installer/steamcmd.zip"


class SteamDownloader:
    @staticmethod
    def download_steamcmd_windows(path):
        download_path = path / "steamcmd.zip"
        max_retries = 10
        print("Installing SteamCMD")
        for count in range(max_retries + 1):
            try:
                urllib.request.urlretrieve(STEAMCMD_WINDOWS_URL, download_path)
            except urllib.error.URLError as e:
                if count < max_retries:
                    continue
                raise e

        print("Extracting steamcmd.zip...")
        with zipfile.ZipFile(download_path, "r") as zr:
            zr.extractall(path)

        os.chdir(path)
        try:
            for n in util.execute("steamcmd +login anonymous +quit"):
                print(n, end="")
        except subprocess.CalledProcessError:
            pass

    @staticmethod
    def find_path():
        home_path = None
        mod_path = None
        try:
            for d in Path(tempfile.gettempdir()).iterdir():
                if d.name[0:4] == "rmm-" and d.is_dir() and (d / ".rmm").is_file():
                    home_path = d
                    break
        except FileNotFoundError:
            pass

        if util.platform() == "win32":
            home_path = Path(tempfile.gettempdir()) / "rmm"
            home_path.mkdir(parents=True, exist_ok=True)

        if not home_path:
            home_path = Path(tempfile.mkdtemp(prefix="rmm-"))
            with open((home_path / ".rmm"), "w"):
                pass

        if not home_path:
            raise Exception("Error could not get temporary directory")

        if util.platform() == "win32":
            mod_path = home_path / "SteamApps/workshop/content/294100/"
        elif util.platform() == "darwin":
            mod_path = (
                home_path
                / "Library/Application Support/Steam/SteamApps/workshop/content/294100/"
            )
        else:
            mod_path = home_path / util.extract_download_path()
        return (home_path, mod_path)

    @staticmethod
    def download(mods: List[int]) -> Tuple[List[Mod], Path]:
        home_path, mod_path = SteamDownloader.find_path()

        if not home_path:
            raise Exception("Error could not get temporary directory")

        workshop_item_arg = " +workshop_download_item 294100 "
        if util.platform() == "win32":
            os.chdir(home_path)
            if not (home_path / "steamcmd.exe").exists():
                SteamDownloader.download_steamcmd_windows(home_path)

            query = "steamcmd +login anonymous {} +quit".format(
                workshop_item_arg + workshop_item_arg.join(str(m) for m in mods),
            )
            print()
            for n in util.execute(query):
                print(n, end="")
        else:
            query = 'env HOME="{}" steamcmd +login anonymous {} +quit >&2'.format(
                str(home_path),
                workshop_item_arg + workshop_item_arg.join(str(m) for m in mods),
            )
            util.run_sh(query)

        # TODO: ugly work around for weird steam problem
        if util.platform() == "linux" and not mod_path.exists():
            mod_path = SteamDownloader.replace_path(mod_path)

        return (ModFolder.read(mod_path), mod_path)

    @staticmethod
    def replace_path(path):
        path_parts = []
        found = False
        for n in reversed(path.parts):
            if n == ".steam" and found == False:
                path_parts.append("Steam")
                found = True
            else:
                path_parts.append(n)

        return Path(*reversed(path_parts))


class WorkshopResult:
    def __init__(
        self,
        steamid,
        name=None,
        author=None,
        description=None,
        update_time=None,
        size=None,
        # num_rating=None,
        rating=None,
        create_time=None,
        num_ratings=None,
    ):
        self.steamid = steamid
        self.name = name
        self.author = author
        self.description = description
        self.update_time = update_time
        self.size = size
        self.create_time = create_time
        self.num_ratings = num_ratings
        self.rating = rating

    def __str__(self):
        return "\n".join(
            [
                prop + ": " + str(getattr(self, prop))
                for prop in self.__dict__
                if not callable(self.__dict__[prop])
            ]
        )

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, WorkshopResult):
            raise NotImplementedError
        return self.steamid == other.steamid


class WorkshopWebScraper:
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36"
    }
    index_query = (
        "https://steamcommunity.com/workshop/browse/?appid=294100&searchtext={}"
    )
    detail_query = "https://steamcommunity.com/sharedfiles/filedetails/?id={}"

    @classmethod
    def _request(cls, url: str, term: str):
        max_retries = 5
        for n in range(max_retries + 1):
            try:
                return urllib.request.urlopen(
                    urllib.request.Request(
                        url.format(term.replace(" ", "+")),
                        headers=WorkshopWebScraper.headers,
                    )
                )
            except urllib.error.URLError as e:
                if n < max_retries:
                    continue
                raise e

    @classmethod
    def detail(cls, steamid: int) -> WorkshopResult:
        results = BeautifulSoup(
            cls._request(cls.detail_query, str(steamid)),
            "html.parser",
        )

        details = results.find_all("div", class_="detailsStatRight")
        try:
            size = details[0].get_text()
        except IndexError:
            size = None
        try:
            created = details[1].get_text()
        except IndexError:
            created = None
        try:
            updated = details[2].get_text()
        except IndexError:
            updated = None
        try:
            description = results.find("div", class_="workshopItemDescription")
            if description:
                description = description.get_text()
        except AttributeError:
            description = None
        try:
            num_ratings = results.find("div", class_="numRatings")
            if num_ratings:
                num_ratings = num_ratings.get_text()
        except AttributeError:
            num_ratings = None
        try:
            rating = re.search(
                "([1-5])(?:-star)",
                str(results.find("div", class_="fileRatingDetails").img),
            )
            if rating:
                rating = rating.group(1)
        except AttributeError:
            rating = None

        return WorkshopResult(
            steamid,
            size=size,
            create_time=created,
            update_time=updated,
            description=description,
            # num_rating=num_ratings,
            rating=rating,
        )

    @classmethod
    def search(cls, term: str, reverse: bool = False) -> List[WorkshopResult]:
        page_result = BeautifulSoup(
            cls._request(cls.index_query, term),
            "html.parser",
        ).find_all("div", class_="workshopItem")

        results = []
        for r in page_result:
            try:
                item_title = r.find("div", class_="workshopItemTitle").get_text()
                author_name = r.find("div", class_="workshopItemAuthorName").get_text()[
                    3:
                ]
                steamid = int(
                    re.search(r"\d+", r.find("a", class_="ugc")["href"]).group()
                )
            except (AttributeError, ValueError):
                continue
            results.append(WorkshopResult(steamid, name=item_title, author=author_name))

        if reverse:
            return list(reversed(results))
        return results



#!/usr/bin/env python3

from __future__ import annotations


from dataclasses import dataclass, field
from multiprocessing import Pool
from pathlib import Path
from typing import Optional, List, Union, cast

import xml.etree.ElementTree as ET

from . import util

DEBUG = False


@dataclass
class Mod:
    packageid: Optional[str] = field(default_factory=str)
    before: List[str] = field(default_factory=list)
    after: List[str] = field(default_factory=list)
    incompatible: List[str] = field(default_factory=list)
    dirname: Optional[Path] = None
    author: str = "Unknown"
    name: Optional[str] = None
    versions: List[str] = field(default_factory=list)
    steamid: Optional[int] = None
    ignored: bool = False
    repo_url: Optional[str] = None
    workshop_managed: Optional[bool] = None
    enabled: Optional[bool] = None

    def title(self) -> str:
        return self.packageid or f"{self.name} by {self.author}"

    def __post_init__(self):
        if self.packageid:
            self.packageid = self.packageid.lower()
        if self.before:
            self.before = [item.lower() for item in self.before]
        if self.after:
            self.after = [item.lower() for item in self.after]

    def __eq__(self, other: Union["Mod", str, int]):
        if isinstance(other, Mod):
            return (self.packageid and self.packageid == other.packageid) or (
                self.steamid and self.steamid == other.steamid
            )
        if isinstance(other, str) and self.packageid:
            return self.packageid.lower() == other.lower()
        if isinstance(other, int) and self.steamid:
            return self.steamid == other
        return NotImplemented

    def __hash__(self):
        if not self.packageid:
            raise ValueError("InvalidPackageHash")
        return hash(self.packageid)

    @staticmethod
    def create_from_workshop_result(wr: "WorkshopResult") -> "Mod":
        return Mod(steamid=wr.steamid, name=wr.name, author=wr.author)

    @staticmethod
    def list_to_dict(mods: List["Mod"]) -> dict:
        return {mod.packageid: mod for mod in mods if mod.packageid}

    @staticmethod
    def create_from_path(path) -> Optional[Mod]:
        def find_about_xml(path: Path) -> Optional[Path]:
            # Use glob to get the case-insensitive About.xml path
            matches = list(path.glob("About/[Aa][Bb][Oo][Uu][Tt].[Xx][Mm][Ll]"))
            return matches[0] if matches else None

        def parse_tree(file_path: Path) -> ET.ElementTree:
            return ET.parse(file_path)

        def extract_packageid(root: ET.Element) -> Optional[str]:
            try:
                return cast(str, cast(ET.Element, root.find("packageId")).text).lower()
            except AttributeError:
                name = util.element_grab("name", root)
                author = util.element_grab("author", root)
                if name and author:
                    return "{}.{}".format(name.lower(), author.lower())
            if DEBUG:
                raise AttributeError("Could not find or infer package ID")
            return None

        def read_steamid(path: Path) -> Optional[int]:
            try:
                file_content = (
                    (path / "About" / "PublishedFileId.txt").read_text().strip()
                )
                return int(file_content.encode("ascii", errors="ignore").decode())
            except (OSError, ValueError, IOError):
                if DEBUG:
                    print(f"Error reading steamid from {path}")
                return None

        def read_ignored(path: Path) -> bool:
            try:
                return (path / ".rmm_ignore").is_file()
            except OSError as e:
                print(e)
                return False

        try:
            about_xml_path = find_about_xml(path)
            if not about_xml_path:
                if not "Place mods here.txt" in str(path):
                    print(f"No About.xml found in {path}")
                return None

            tree_root = parse_tree(about_xml_path)
            package_id = extract_packageid(tree_root)
            if not package_id:
                return None

            author = util.element_grab("author", tree_root)
            if not author:
                author = util.list_grab("author", tree_root)
            if isinstance(author, list):
                author = ", ".join(author)
            if not author:
                author = "Unknown"

            return Mod(
                packageid=package_id,
                before=util.list_grab("loadAfter", tree_root),
                after=util.list_grab("loadBefore", tree_root),
                incompatible=util.list_grab("incompatibleWith", tree_root),
                dirname=path.name,
                author=author,
                name=util.element_grab("name", tree_root),
                versions=util.list_grab("supportedVersions", tree_root),
                steamid=read_steamid(path),
                ignored=read_ignored(path),
            )

        except OSError:
            print(f"Ignoring {path}")
        except ET.ParseError:
            print(f"Ignoring {path}.\n\t{about_xml_path} contains invalid XML.")


class ModFolder:
    @staticmethod
    def read(path: Path) -> list[Mod]:
        with Pool(16) as p:
            mods = cast(
                list[Mod],
                list(
                    filter(
                        None,
                        p.map(
                            Mod.create_from_path,
                            path.iterdir(),
                        ),
                    )
                ),
            )

        return mods

    @staticmethod
    def read_dict(path: Path):
        return Mod.list_to_dict(ModFolder.read(path))

    @staticmethod
    def search(path: Path, search_term) -> list[Mod]:
        return [
            r
            for r in ModFolder.read(path)
            if (isinstance(r.name, str) and search_term.lower() in r.name.lower())
            or (isinstance(r.author, str) and search_term.lower() in r.author.lower())
            or search_term == r.steamid
        ]

    @staticmethod
    def search_dict(path: Path, search_term) -> dict[str, Mod]:
        return Mod.list_to_dict(ModFolder.search(path, search_term))


EXPANSION_PACKAGES = [
    Mod(packageid="ludeon.rimworld", author="Ludeon", name="RimWorld"),
    Mod(packageid="ludeon.rimworld.ideology", author="Ludeon", name="Ideology"),
    Mod(packageid="ludeon.rimworld.royalty", author="Ludeon", name="Royalty"),
    Mod(packageid="ludeon.rimworld.biotech", author="Ludeon", name="Biotech"),
]
